package mocman.interpreter.syntax;

import mocman.interpreter.syntax.token.*;
import mocman.interpreter.*;
import java.util.*;
import mocman.util.*;

public class ParserTable
{
	public static final int POP=0;
	public static final int PARSE_ERROR=-1;
	public static final int EMPTY=-2;
	private static HashMap<Pair<Integer,Integer>,Integer> table= new HashMap<Pair<Integer,Integer>,Integer>();
	public static void insert(int i,int j,int ruleindex)
	{
		//if ((i==100)&&(j==9)) System.out.println("IF "+ruleindex);
		Pair<Integer,Integer> p = new Pair<Integer,Integer>(i,j);
		if (table.get(p)!=null)
		{
			if (!table.get(p).equals(ruleindex))
			{
				System.out.println("ERROR! Conflict in the parser table! `"+Rules.getRuleName(table.get(p))+"' conflicts with `"+Rules.getRuleName(ruleindex)+"' @(`"+Tokenizer.getTokenName(i)+"',`"+Tokenizer.getTokenName(j)+"')");
			}
			return;
		}
//		System.out.println("OK, inserted rule no. " + ruleindex + " at " + i +":"+ j);
		table.put(p,ruleindex);
	}
	public static String getS()
	{
		return table.toString();
	}
	public static String getHTML()
	{
		StringBuilder s=new StringBuilder();
		s.append("<table style=\"border:1px solid black\">");
		s.append("<tr><td style=\"border:1px solid black\">&nbsp;</td>");
		for (int i=0;i<Tokenizer.getPatternCount();i++)
		{
			if (Rules.isTerminal(Tokenizer.getPattern(i).getType()))
			{
				s.append("<td style=\"border:1px solid black\">");
				s.append(Tokenizer.getPattern(i).getCode());
				s.append("</td>");
			}
		}		
		s.append("</tr>");
		int c=0;
		for (int i=0;i<Tokenizer.getPatternCount();i++)
		{
			if (!Rules.isTerminal(Tokenizer.getPattern(i).getType()))
			{
				c++;
				s.append("<tr style=\"background-color:");
				if (c%2==0) s.append("rgb(240,220,220);");
				else s.append("rgb(220,220,240);");
				s.append("\"><td style=\"border:1px solid black\">");
				s.append(Tokenizer.getPattern(i).getCode());
				s.append("</td>");
				for (int j=0;j<Tokenizer.getPatternCount();j++)
				{
					if (Rules.isTerminal(Tokenizer.getPattern(j).getType()))
					{
						s.append("<td style=\"border:1px solid black\">");
						Pair<Integer,Integer> p = new Pair<Integer,Integer>(new Integer(Tokenizer.getPattern(i).getType()),new Integer(Tokenizer.getPattern(j).getType()));
						if ((table.get(p)==null)||(table.get(p)<0)) 
						{
							s.append("&nbsp;");
						}
						else 
						{
							s.append("(");
							s.append(Rules.getRight(table.get(p)));
							s.append(",");
							s.append(table.get(p));
							s.append(")");
						}
						s.append("</td>");
					}
				}
				s.append("</tr>");
			}
		}		
		s.append("</table>");
		return s.toString();
	}
	public static int get(int i,int j)
	{
		Pair<Integer,Integer> p = new Pair<Integer,Integer>(i,j);
		if (table.get(p)!=null) return table.get(p);
		return EMPTY;
	}
	public static int nextRule(int term,int rule)
	{
		if (term==rule) return POP; //POP
		Pair<Integer,Integer> p = new Pair<Integer,Integer>(term,rule);
		if (table.get(p)==null) return PARSE_ERROR; //ERROR
		return table.get(p);
	}
	public static int getHash()
	{
		return table.hashCode();
	}
}
