package mocman.interpreter.syntax;

import mocman.interpreter.syntax.token.*;

public class PassMatrix implements Cloneable
{
	private int[][] pm;
	private int n;
	public PassMatrix()
	{
		n=Tokenizer.getPatternCount();
		pm=new int[n][n];
	}

	public PassMatrix clone()
	{
		PassMatrix other = new PassMatrix();
		other.pm=pm.clone();
		return other;
	}

	public PassMatrix mul(PassMatrix other)
	{
		PassMatrix x=new PassMatrix();
		for (int i=0;i<n;i++)
		{
			for (int j=0;j<n;j++)
			{
				x.pm[i][j]=0;
			}
		}
		for (int j=0;j<n;j++)
		{
			for (int i=0;i<n;i++)
			{
				if (pm[i][j]==1)
				{
					for (int k=0;k<n;k++)
					{
						if (other.pm[j][k]==1)
						{
							x.pm[i][k]=1;
						}
					}
				}
			}
		}
		return x;
	}

	public void set(int i,int j)
	{
		pm[Tokenizer.getPatternId(i)][Tokenizer.getPatternId(j)]=1;
	}

	public PassMatrix refl()
	{
		for (int i=0;i<n;i++)
		{
			pm[i][i]=1;
		}
		return this;
	}

	public PassMatrix closure()
	{
		//System.out.println("closure");
		int h=hashCode();
		pm=mul(this).pm;
		if (h!=hashCode()) closure();
		return this;
	}

	public String toString()
	{
		StringBuilder s=new StringBuilder();
		s.append("<table><tr><td>&nbsp;</td>");
		for (int i=0;i<n;i++)
		{
			s.append("<td style=\"border:1px solid black\">");
			s.append(i);
			s.append("</td>");
		}
		s.append("</tr>");
		for (int i=0;i<n;i++)
		{
			s.append("<tr><td>");
			s.append(i);
			s.append("</td>");
			for (int j=0;j<n;j++)
			{
				s.append("<td style=\"border:1px solid black\">");
				if(pm[j][i]==1) s.append(1); else s.append("&nbsp;");
				s.append("</td>");
			}
			s.append("</tr>");
		}
		s.append("</table>");
		return s.toString();
	}

	public int hashCode()
	{
		int c=0;
		for (int i=0;i<n;i++)
		{
			for (int j=0;j<n;j++)
			{
				if(pm[i][j]==1) c=c+(i+1)*(j+1);
			}
		}
		return c;
	}

	public int get(int i,int j)
	{
		return pm[i][j];
	}
}
