package mocman.interpreter.syntax;

import mocman.interpreter.syntax.token.*;
import mocman.interpreter.syntax.*;
import java.util.*;
import mocman.util.*;

public class SyntaxTree
{
	public SyntaxTree parent;
	public Vector<SyntaxTree> Leaf;
	public int at=0;
	public int numLeaves=-1;
	private String value;
	private int intv;
	private Token token;
	public ITreeNode node;
	public SyntaxTree()
	{
		Leaf = new Vector<SyntaxTree>();
	}
	public SyntaxTree addChild(int t)
	{
		SyntaxTree child = new SyntaxTree();
		Leaf.addElement(child);
		child.value=(new Integer(t)).toString();
		child.intv=t;
		child.parent=this;
		child.node=new Local();
	//	child.node.build(Tokenizer.getTokenName(child.intv),0);
		numLeaves++;
		return child;
	}
	public void setNodeRule(int i)
	{
		node.setRule(i);
	}
	private SyntaxTree getNext()
	{
		if (Leaf.size()<=at) 
		{
			if (parent==null) return null;
			return parent.getNext();
		}
		else
		{
			at=at+1;
			return Leaf.get(at-1);
		}
	}
	public static SyntaxTree build(Vector<Integer> list,Stack<Token> tokens)
	{
		SyntaxTree builder=new SyntaxTree();
		builder.node=new Local();
		builder.value="Root";
		builder.intv=-4;
		SyntaxTree root=builder;
		for (int i=0;i<list.size();i++)
		{
			int k=list.get(i);
			builder.setNodeRule(k);
			Pair<Integer,Vector<Integer>> rule = Rules.getRule(k);
			for (int j=0;j<rule.getValue().size();j++)
			{
				builder.addChild(rule.getValue().get(j));
			}
			builder=builder.getNext();
			if (builder==null) return root;
			while (Rules.isTerminal(builder.intv)||(Tokenizer.epsilon()==builder.intv))
			{
				if ((Tokenizer.epsilon()==builder.intv))
				{
					builder.node.setValue("epsilon");
				}
				else
				{
					builder.token=tokens.pop();
					builder.node.setValue(builder.token.getCode());
				}
				builder=builder.getNext();
			if (builder==null) return root;
			}
		}
		return root;
	}
	public String terms()
	{
		StringBuilder sb=new StringBuilder();
		if (token!=null)
		{
			sb.append(token.getCode());
		}
		else
		{
			//sb.append(Tokenizer.getTokenName(intv));
			sb.append("(");
			for (int i=0;i<=numLeaves;i++)
			{
				if (i>0) sb.append(",");
				sb.append(Leaf.get(i).terms());
			}
			sb.append(")");
		}
		return sb.toString();
	}
	public String toString()
	{
		StringBuilder sb=new StringBuilder();
		if (token!=null)
		{
			sb.append(token.getCode());
		}
		else
		{
			sb.append(Tokenizer.getTokenName(intv));
			sb.append("(");
			for (int i=0;i<=numLeaves;i++)
			{
				if (i>0) sb.append(",");
				sb.append(Leaf.get(i));
			}
			sb.append(")");
		}
		return sb.toString();
	}
/*	SyntaxTree(String s)
	{
		build(s);
	}
	private Kind parse(String s, String sl, String sr)
	{
		Tokenizer t = new Tokenizer();
		t.tokenize(s);
		
		//TODO: elemzés
		//TODO: kind generálás
		//TODO: szemantikai paraméterek
		return new Kind(Kind.Type.local);
	}

	public void build(String s)
	{
		//TODO: szintaxisfa generálása
		String s1="";
		String s2="";
		Kind k = parse(s,s1,s2);
		left = new SyntaxTree(s1);
		right = new SyntaxTree(s2);
		//Alapesetben Local-t generálunk.
		node = new Local();
		switch (k.get())
		{
			case global: node = new Global(); break;
			case script: node = new Script(s); break;
			case module: node = new Module(s); break;
		}
		node.build(s);
	}*/
	public int getChild()
	{
		return Leaf.get(0).Leaf.get(0).intv;
	}
	public ITreeNode run()
	{
		ITreeNode[] k=new ITreeNode[Leaf.size()];
		//System.out.println(Tokenizer.getTokenName(intv));
		for (int i=0;i<Leaf.size();i++)
		{
			k[i]=Leaf.get(i).run();
			//System.out.println(node.getVal()+"@"+i+" -> "+k[i].getVal());
		}
		ITreeNode n=Generated.semantic(node,k);
		//System.out.println(n.getVal());
		return n;
	}
}
