package mocman.interpreter.syntax.token;

import java.util.*;
import java.util.regex.*;
import mocman.interpreter.syntax.Rules;
public class Tokenizer
{
	private Vector<Token> tokens;
	private static Vector<Token> patterns= new Vector<Token>();
	private int tokencount;
	private int where;
	private static Integer patterncount = new Integer(0);
	private int matches;
	private Token pmatch;
	private static int eps=-1;


	public Tokenizer()
	{
		tokens=new Vector<Token>();
	}

	public static String getTokenName(int t)
	{
		if (t==-4) return "Root";
		for (int i=0;i<patterns.size();i++)
		{
			if (patterns.get(i).getType().equals(t))
			{
				return patterns.get(i).getCode();
			}
		}
		return "No such token";
	}

	public static int getPatternId(int p)
	{
		for (int i=0;i<patterns.size();i++)
		{
			if (patterns.get(i).getType()==p) return i;
		}
		return -1;
	}	

	public static int epsilon()
	{
		if (eps==-1)
		{
			for (int i=0;i<patterns.size();i++)
			{
				if (patterns.get(i).getCode().equals("epsilon"))
				{
					eps=patterns.get(i).getType();
					return eps;
				}
			}
		}
		return eps;
	}

	public static int getPatternCount()
	{
		return patterns.size();
	}

	public static Token getPattern(int i)
	{
		return patterns.get(i);
	}

	public static void addTokenType(String pattern,int type)
	{
		patterns.add(patterncount, new Token(pattern,type));
		patterncount++;
	}
	public void empty()
	{
		tokens.clear();
		where=0;
		tokencount=0;
	}
	private boolean match(String s1,String s2)
	{
		return (Pattern.compile(s1).matcher(s2).matches());
	}
	public String match(String w)
	{
		for (int i=0;i<patterncount;i++)
		{
			if (Rules.isTerminal(patterns.get(i).getType()))
			{
				String pat="^\\s*("+patterns.get(i).getCode()+")\\s*(.*)";
				if (match(pat,w))
				{
					//System.out.println(w.replaceAll(pat,"$1"));
					tokens.addElement(new Token(w.replaceAll(pat,"$1"),patterns.get(i).getType()));
					tokencount++;
					return w.replaceAll(pat,"$2");
				}
				else
				{
					//System.out.println("No match: "+pat+" against "+w); 
				}
			}
		}
		return "";
	}
	public void tokenize(String s)
	{
		String w =s;
		empty();
		while (w!="")
		{
//			System.out.println(w);
			w=match(w);	
		}
/*		String w="";
		for (int i=0;i<s.length();i++)
		{
			match(w);
			if (matches==0)
			{
				//too much whitespace will also create invalid tokens
				//we should not let the invalid tokens to cause trouble
				if (pmatch.getType()!=Token.INVALID_TOKEN)
				{
					tokens.set(tokencount,pmatch);
					tokencount++;
					w=w+s.charAt(i);
				}
			}
			else
			{
				w=w+s.charAt(i);
			}
		}*/
	}
	public Token getNext()// throws ArrayIndexOutOfBoundsException (should not throw when used properly)
	{
		where++;
		return tokens.get(where-1);
	}
	public boolean isComplete()
	{
		return where >= tokencount;
	}
	public boolean hasNext()
	{
		return !isComplete();
	}
}
