package mocman.util.awt;

import mocman.util.data.*;
import java.awt.*;
import javax.swing.*;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

/**
 * Generic Graph type on AWT platform
 * It contains a generic graph (Graph<TN,TE>).
 * It is designed to be shown on an awt from.
 * @author <a href="mailto:mocman@inf.elte.hu">Tibor Zoltan Varkonyi</a>
 * @param TN value type of the nodes
 * @param TE value type of the edges
 */
public class GraphPrinter<TN,TE> extends Canvas 
{
	private Graph<TN,TE> graph;
	private int nodewidth;
	private int ww;
	private int wh;
	private boolean autosize;
	private boolean modw;
	public int selectedNode;
	public int selectedEdge;
/**
 * Sets the automatic sizing feature
 */
	public void setAutoSize(boolean a)
	{
		autosize=a;
	}

/**
 * Gets the automatic sizing feature
 */
	public boolean getAutoSize()
	{
		return autosize;
	}
/**
 * Sets the width of the control
 */
	public void setWidth(int w)
	{
		ww=w-20;
		setSize(ww+20, wh+20);
	}

/**
 * Sets the height of the control
 */
	public void setHeight(int h)
	{
		wh=h-20;
		setSize(ww+20, wh+20);
	}

/**
 * Gets the width of the control
 */
	public int getWidth()
	{
		return ww+20;
	}

/**
 * Gets the height of the control
 */
	public int getHeight()
	{
		return wh+20;
	}

/**
 * Default constructor
 */
	public GraphPrinter() 
	{
		autosize=true;
		graph = new Graph<TN,TE>();
		ww=20;wh=20;
		modw=false;
		setSize(ww+20, wh+20);
		nodewidth=25; 
	}

/**
 * Sets the minimal width of a node.
 * The nodes are automatically resized if data length is bigger than the node.
 */
	public void setNodeWidth(int i)
	{
		nodewidth=i;
	}

/**
 * Gets the minimal node width
 */
	public int getNodeWidth()
	{
		return nodewidth;
	}

/**
 * Sets the graph
 */
	public void setGraph(Graph<TN,TE> g)
	{
		graph = g;
	} 

/**
 * Gets the graph
 */
	public Graph<TN,TE> getGraph()
	{
		return graph;
	}

/**
 * The paint method for the awt form
 */
	public void paint(Graphics g1) 
	{
	    Graphics2D g = (Graphics2D) g1;
	    BufferedImage image = // Create an off-screen image
   	 	new BufferedImage(ww+20, wh+20, BufferedImage.TYPE_INT_RGB);
		Graphics2D ig = image.createGraphics(); // Get its Graphics for drawing
		ig.setRenderingHint(RenderingHints.KEY_ANTIALIASING, // Anti-alias!
        RenderingHints.VALUE_ANTIALIAS_ON);
		ig.setColor(java.awt.Color.lightGray);
		ig.fill3DRect(0, 0, ww+19, wh+19,true); // draw border 
		ig.setColor(java.awt.Color.black);
		for (int i=0;i<graph.edgeCount();i++)
		{
			try
			{
				int wx=g.getFontMetrics().stringWidth(graph.edges(i).getValue().toString());
				int wy=5;//g.getFontMetrics().stringHeight(graph.edges(i).getValue().toString());
				int x1=graph.nodes(graph.edges(i).getX()).getX();
				int y1=graph.nodes(graph.edges(i).getX()).getY();
				int x2=graph.nodes(graph.edges(i).getY()).getX();
				int y2=graph.nodes(graph.edges(i).getY()).getY();
				if (selectedEdge==i) 
				{
					ig.setStroke(new BasicStroke(5.0f));
					ig.setColor(Color.blue.brighter());
					ig.drawLine( x1,y1,x2,y2);
					ig.setStroke(new BasicStroke(3.0f));
					ig.setColor(Color.blue.darker());
					ig.drawLine( x1,y1,x2,y2);
				}
				ig.setStroke(new BasicStroke(1.5f));
				ig.setColor(graph.edges(i).getColor());
				ig.drawLine( x1,y1,x2,y2);
				ig.setColor(java.awt.Color.lightGray);
				ig.setColor(java.awt.Color.black);
				ig.drawString(graph.edges(i).getValue().toString(), (x1+x2*2)/3-wx/2,(y1+y2*2)/3+wy/2); 
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		ig.setStroke(new BasicStroke(1.0f));
		for (int i=0;i<graph.nodeCount();i++)
		{
			int wx=g.getFontMetrics().stringWidth(graph.nodes(i).getValue().toString());
			int wk=wx/2+5;
			if (wk<nodewidth) wk=nodewidth;
			ig.setColor(graph.nodes(i).getColor());
			//ig.fillOval(graph.nodes(i).getX()-wk,graph.nodes(i).getY()-wk,wk*2,wk*2);
			ig.fillRect(graph.nodes(i).getX()-wk,graph.nodes(i).getY()-10,wk*2,20);
			ig.setColor(java.awt.Color.black);
			//ig.drawOval(graph.nodes(i).getX()-wk,graph.nodes(i).getY()-wk,wk*2,wk*2);
			if (selectedNode==i) ig.setColor(java.awt.Color.blue);
			ig.drawRect(graph.nodes(i).getX()-wk,graph.nodes(i).getY()-10,wk*2,20);
			if (graph.nodes(i).getX()+wk>ww)
			{
				ww=graph.nodes(i).getX()+wk;
				modw=true;
			}
			if (graph.nodes(i).getY()+wk>wh)
			{
				wh=graph.nodes(i).getY()+wk;
				modw=true;
			}
			ig.setColor(java.awt.Color.white);
			ig.setColor(java.awt.Color.black);
			ig.drawString(graph.nodes(i).getValue().toString(), graph.nodes(i).getX()-wx/2,graph.nodes(i).getY()+5); 
		}
	    g.drawImage(image, 0, 0, this);
		if (autosize)
		{
			if (modw) 
			{
				setSize(ww+20,wh+20);
			}
		}
		modw=false;
	} 

	public void sizer()
	{
		for (int i=0;i<graph.nodeCount();i++)
		{
			int wx=10;//g.getFontMetrics().stringWidth(graph.nodes(i).getValue().toString());
			int wk=wx/2+5;
			if (wk<nodewidth) wk=nodewidth;
			if (graph.nodes(i).getX()+wk>ww)
			{
				ww=graph.nodes(i).getX()+wk;
				modw=true;
			}
			if (graph.nodes(i).getY()+wk>wh)
			{
				wh=graph.nodes(i).getY()+wk;
				modw=true;
			}
		}
		if (autosize)
		{
			if (modw) 
			{
				setSize(ww+20,wh+20);
			}
		}
		modw=false;
	}

/**
 * update method for double buffering
 */
	public void update(Graphics g) 
	{
		paint(g);
	}
} 
